@@ -20,7 +20,7 @@ compensates for some of the idiosyncrasies of SQLite.
 
 =head1 METHODS
 
-=head2
+=head2 database_version
 
 Returns the version of the SQLite library which is used, e.g., "2.8.0".
 SQLite can only return short variant.
@@ -71,10 +71,6 @@ sub case_sensitive {
     return (1);
 }
 
-sub binary_safe_blobs {
-    return undef;
-}
-
 =head2 distinct_count STATEMENTREF
 
 takes an incomplete SQL SELECT statement and massages it to return a DISTINCT result count
